-- ========================================================
--                          Data
-- ========================================================

local info = {
	-- Default entry
	[0] = {
		absorbType = 1, -- 1: normal,  0: disable, -1: necrotic
		critType = nil, -- 1: magical, 0: disable, -1: physical, nil: default
	},

	-- Player spells
	[1454] = { absorbType = -1 }, -- Life Tap (glyphed)
	[48743] = { absorbType = -1 }, -- Death Pact
	[116888] = { absorbType = -1 }, -- Purgatory
	[219521] = { absorbType = -1 }, -- Shadow Covenant
	[110913] = { absorbType = 0, onDurationFound = function(effect, unit) effect.absorb = math.huge; end }, -- Dark Bargain
	[125174] = { absorbType = 0, onDurationFound = function(effect, unit) effect.absorb = UnitHealthMax(unit) / 2; end }, -- Touch of Karma

	-- PvE spells
	[66237] = { absorbType = -1 }, -- Incinerate Flesh (LK, Jaraxxus)
	[105479] = { absorbType = -1 }, -- Searing Plasma (Cataclysm, Deathwing Spine)
	[107140] = { absorbType = -1 }, -- Magnetic Shroud (MoP, Pandashan Monastery)
	-- TODO: maybe others?
}

local effects = { };

local EMPTY_TABLE = { };

-- Debug: /dump _EFFECTS
-- _EFFECTS = effects;

-- ========================================================
--                         Functions
-- ========================================================

function OverTimePredictor_GetSpellInfo(spellId)
	return info[spellId] or info[0];
end

function OverTimePredictor_GetEffectiveSpellId(spellId, sourceGUID, destGUID) -- return spellId
	if spellId == 122470 and sourceGUID == destGUID then
		return 125174; -- Combat log says you gain Touch of Karma but the buff spellId is different... So a hack just for this spell...
	end
	return spellId;
end

-- Initially crit ticks / normal ticks were sampled and a critical rate was derived, but it was useless (i.e: zero) for PvE DoTs and long to build up in PvP to be accurate.
-- Critical ticks remain an unpredictable thing in PvP, so the following compromise has been elaborated to avoid wasting memory with dubious and costly sampling:
-- Crit rate is set to zero for all entities beside the local player, and the local player uses his known physical crit for physical school DoTs and magical crit for the other DoTs.

function OverTimePredictor_GetCritRate(spellId, sourceGUID, school, powerType) -- return critRate
	local critRate = 0;
	if powerType < 0 and sourceGUID == UnitGUID("player") and OverTimePredictor_Options["PredictCrit"] then
		local info = OverTimePredictor_GetSpellInfo(spellId);
		if not info.critType then
			-- Automatic mode
			if school == SCHOOL_MASK_NONE or school == SCHOOL_MASK_PHYSICAL then
				critRate = GetCritChance();
			else
				critRate = GetSpellCritChance(2);
			end

		elseif info.critType < 0 then
			-- Physical crit
			critRate = GetCritChance();

		elseif info.critType > 0 then
			-- Magical crit
			critRate = GetSpellCritChance(2);
		end
	end
	return critRate / 100;
end

function OverTimePredictor_GetNumEffects(destGUID) -- return count
	return #(effects[destGUID] or EMPTY_TABLE);
end

function OverTimePredictor_GetEffect(destGUID, argument, spellId) -- arguments: (destGUID, index) or (destGUID, sourceGUID, spellId), return effect, index
	local unit_effects = effects[destGUID];
	if unit_effects then
		if type(argument) == "number" then
			return unit_effects[argument], argument;
		else
			for i=1, #unit_effects do
				if unit_effects[i].source == argument and unit_effects[i].spell == spellId then
					return unit_effects[i], i;
				end
			end
		end
	end
	return nil, 0;
end

function OverTimePredictor_AddEffect(destGUID, effect) -- return index
	local unit_effects = effects[destGUID];
	if not unit_effects then
		unit_effects = OverTimePredictor_CreateTable();
		effects[destGUID] = unit_effects;
	end
	tinsert(unit_effects, effect);
	return #unit_effects;
end

function OverTimePredictor_RemoveEffect(destGUID, index) -- return success
	local unit_effects = effects[destGUID];
	if unit_effects and index >= 1 and index <= #unit_effects then
		OverTimePredictor_DeleteTable(unit_effects[index]);
		tremove(unit_effects, index);
		if #unit_effects == 0 then
			OverTimePredictor_DeleteTable(unit_effects);
			effects[destGUID] = nil;
		end
		return true;
	end
	return false;
end

function OverTimePredictor_WipeDatabase()
	wipe(effects);
end